local addon = LibStub("AceAddon-3.0"):NewAddon("AO_BossMod", "AceEvent-3.0")
local version = "3.1.0"

local addonPrefix = "AOBossMod"
local lastPlayedTime = {}
local silenceDelay = 4

local debugMode = false

-- Bibliothèque de sons
local mediaPath = "Interface\\AddOns\\ao_bossmod\\medias\\"
local soundsTable = {
    "sleep_canister.ogg",
    "ruiner.ogg",
    "apocalypse_protocol.ogg",
    "storm_of_darkness.ogg",
    "torment.ogg",
    "taeshalach.ogg",
    "cone_of_death.ogg",
    "soul_bomb.ogg"
}

-- Menu de configuration
local soundToTest
local options = {
    type = "group",
    args = {
        enable = {
            name = "Activer AO BossMod",
            desc = "Activer / désactiver AO BossMod",
            type = "toggle",
            width = "full",
            set = function(info, val) addon.db.profile.enabled = val end,
            get = function(info) return addon.db.profile.enabled end
        },
        soundChannel = {
            name = "Canal",
            desc = "Canal audio pour les annonces vocales et les alertes",
            type = "select",
            order = 1,
            values = {
                Master = "Master",
                SFX = "SFX",
                Ambience = "Ambience",
                Music = "Music"
            },
            set = function(info, val) addon.db.profile.soundchannel = val end,
            get = function(info) return addon.db.profile.soundchannel end
        },
        testVoice = {
            name = "Test",
            desc = "Tester les voix (sélectionner une alerte, puis cliquer sur GO!)",
            type = "select",
            order = 2,
            values = soundsTable,
            set = function(info, val) soundToTest = val end,
            get = function(info) return soundToTest end
        },
        goTestVoice = {
            name = "GO!",
            desc = "Ecouter l'alerte",
            type = "execute",
            order = 3,
            --func = function() if soundToTest then	addon:handleRaidInstruction(soundToTest, "Test Button") end end
            func = function() if soundToTest then PlaySoundFile(mediaPath .. soundsTable[soundToTest], addon.db.profile.soundchannel) end end
        },
        imonar = {
            name = "Antorus : Imonar",
            type = "group",
            order = 12,
            args = {
                alertsHeader = {
                    name = "Alertes sonores",
                    type = "description",
                    fontSize = "medium"
                },
                sleep_canister = {
                    name = "Bombes de sommeil",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.sleep_canister = val end,
                    get = function(info) return addon.db.profile.sleep_canister end
                }
            }
        },
        kingaroth = {
            name = "Antorus : Kin'Garoth",
            type = "group",
            order = 13,
            args = {
                alertsHeader = {
                    name = "Alertes sonores",
                    type = "description",
                    fontSize = "medium"
                },
                ruiner = {
                    name = "Dévastateur",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.ruiner = val end,
                    get = function(info) return addon.db.profile.ruiner end
                },
                apocalypse_protocol = {
                    name = "Protocole d'Apocalypse",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.apocalypse_protocol = val end,
                    get = function(info) return addon.db.profile.apocalypse_protocol end
                }
            }
        },
        shivarra = {
            name = "Antorus : Sabbat des shivarra",
            type = "group",
            order = 14,
            args = {
                alertsHeader = {
                    name = "Alertes sonores",
                    type = "description",
                    fontSize = "medium"
                },
                storm_of_darkness = {
                    name = "Tempête de ténèbres",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.storm_of_darkness = val end,
                    get = function(info) return addon.db.profile.storm_of_darkness end
                },
                torment = {
                    name = "Tempête de ténèbres",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.torment = val end,
                    get = function(info) return addon.db.profile.torment end
                }
            }
        },
        aggramar = {
            name = "Antorus : Aggramar",
            type = "group",
            order = 15,
            args = {
                alertsHeader = {
                    name = "Alertes sonores",
                    type = "description",
                    fontSize = "medium"
                },
                taeshalach = {
                    name = "Taeshalach",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.taeshalach = val end,
                    get = function(info) return addon.db.profile.taeshalach end
                }
            }
        },
        argus = {
            name = "Antorus : Argus",
            type = "group",
            order = 16,
            args = {
                alertsHeader = {
                    name = "Alertes sonores",
                    type = "description",
                    fontSize = "medium"
                },
                cone_of_death = {
                    name = "Cône de mort",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.cone_of_death = val end,
                    get = function(info) return addon.db.profile.cone_of_death end
                },
                soul_bomb = {
                    name = "Bombe d'esprit",
                    desc = "Activer/désactiver l'alerte",
                    type = "toggle",
                    width = "full",
                    set = function(info, val) addon.db.profile.soul_bomb = val end,
                    get = function(info) return addon.db.profile.soul_bomb end
                }
            }
        }
    }
}

-- Valeurs par défaut
local defaults = {
    profile = {
        sleep_canister = true,
        ruiner = true,
        apocalypse_protocol = true,
        storm_of_darkness = true,
        torment = true,
        taeshalach = true,
        cone_of_death = true,
        soul_bomb = true,
        enabled = true,
        soundchannel = "Master"
    }
}

local function printDebug(text)
    if debugMode then print("[DEBUG][CLIENT] " .. text) end
end

function addon:OnInitialize()
    -- Called when the addon is loaded
    self.db = LibStub("AceDB-3.0"):New("AO_BossModDB", defaults, true)
    LibStub("AceConfig-3.0"):RegisterOptionsTable("AO_BossMod", options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AO_BossMod", "AO BossMod [Player]")
end

function addon:OnEnable()
    printDebug("AO Boss Mod Player init")
    self:RegisterEvent("CHAT_MSG_ADDON", "handleMessageAddon")
    RegisterAddonMessagePrefix(addonPrefix)
end

-- addon:CHAT_MSG_ADDON
-- Executé lors d'un évenement CHAT_MSG_ADDON
-- Vérifie qu'il s'agit d'un message AOBossMod, et que l'émetteur est un assistant ou le raid leader
function addon:handleMessageAddon(event, prefix, msg, channel, sender)
    --printDebug(event..prefix..msg..channel..sender)
    if prefix == addonPrefix and (UnitIsGroupAssistant(sender) or UnitIsGroupLeader(sender)) then
        if strfind(msg, "myAOBossModVersion") then return end -- pour éviter de traiter son propre wisp de réponse...
        if strfind(msg, "getAOBossModVersion") then
            self:handleGetVersion(sender)
        elseif strfind(msg, "takeScreenshot") then
            self:takeScreenshot()
        else
            if (self.db.profile.enabled) then self:handleRaidInstruction(msg, sender) end
        end
    end
end

-- handleRaidInstruction
-- Lance la lecture de l'instruction vocale envoyé par le raid lead
-- Zone de "silence" (variable silenceDelay) après chaque lecture, si jamais plusieurs assistants ont le module raid lead, ça évite de flood.
function addon:handleRaidInstruction(message, sender)

    -- Ici c'est laid, faut refaire un truc mieux que ça bobby...
    if (not self.db.profile.sleep_canister and message == "sleep_canister") then return end
    if (not self.db.profile.ruiner and message == "ruiner") then return end
    if (not self.db.profile.apocalypse_protocol and message == "apocalypse_protocol") then return end
    if (not self.db.profile.storm_of_darkness and message == "storm_of_darkness") then return end
    if (not self.db.profile.torment and message == "torment") then return end
    if (not self.db.profile.taeshalach and message == "taeshalach") then return end
    if (not self.db.profile.cone_of_death and message == "cone_of_death") then return end
    if (not self.db.profile.soul_bomb and message == "soul_bomb") then return end

    printDebug("Receive message " .. message .. " from " .. sender)
    if (lastPlayedTime[message] == nil) or (time() - lastPlayedTime[message] > silenceDelay) then
        lastPlayedTime[message] = time()
        printDebug("Play sound " .. mediaPath .. message .. ".ogg on " .. self.db.profile.soundchannel)
        PlaySoundFile(mediaPath .. message .. ".ogg", self.db.profile.soundchannel)
    end
end

-- handleGetVersion
-- Le raid leader a envoyé une requête pour connaître la version du client
function addon:handleGetVersion(sender)
    printDebug("Receive message getAOBossModVersion from " .. sender)
    printDebug("Send Version " .. version .. " to " .. sender)
    SendAddonMessage(addonPrefix, "myAOBossModVersion " .. version, "WHISPER", sender)
end

function addon:takeScreenshot()
    printDebug("Receive screenshot instruction")
    Screenshot()
end
